@@ -11,6 +11,9 @@ import measure
 SPACES = {}
 
 
+class UserError(Exception):
+    pass
+
 class MetaSpace(type):
 
     def __new__(cls, name, bases, attrs):
@@ -23,36 +26,32 @@ class MetaSpace(type):
             if not type(b) == cls:
                 continue
             if hasattr(b, '_dimensions'):
-                for name, dim in b._dimensions:
-                    attrs[name] = copy(dim)
+                for dim in b._dimensions:
+                    attrs[dim.name] = copy(dim)
 
             if hasattr(b, '_measures'):
-                for name, msr in b._measures:
-                    attrs[name] = copy(msr)
+                for msr in b._measures:
+                    attrs[msr.name] = copy(msr)
 
         dimensions = []
         measures = []
         for k, v in attrs.iteritems():
             # Collect dimensions
             if isinstance(v, dimension.Dimension):
-                dimensions.append((k, v))
-                # TODO use name instead of _name (the _ doesnt make sense )
-                v._name = k
+                dimensions.append(v)
+                v.name = k
 
             # Collect measures
             if isinstance(v, measure.Measure):
-                measures.append((k, v))
-                v._name = k
+                measures.append(v)
+                v.name = k
 
         attrs['_dimensions'] = dimensions
         attrs['_measures'] = measures
-        attrs['_read_cache'] = {}
-        attrs['_insert_cache'] = {}
-        attrs['_update_cache'] = {}
 
         spc = super(MetaSpace, cls).__new__(cls, name, bases, attrs)
 
-        for _, dim in dimensions:
+        for dim in dimensions:
             dim._spc = spc
 
         if bases:
@@ -71,10 +70,10 @@ class Space:
     @contextmanager
     def connect(cls, uri):
         cls._db = backend.get_backend(uri)
-        for _, dim in cls._dimensions:
+        for dim in cls._dimensions:
             dim.set_db(cls._db)
 
-        for _, msr in cls._measures:
+        for msr in cls._measures:
             msr.set_db(cls._db)
 
         cls._db.register(cls)
@@ -85,7 +84,7 @@ class Space:
     def key(cls, point, create=False):
         key = tuple(
             dim.key(point.get(name, tuple()), create=create)
-            for name, dim in cls._dimensions)
+            for dim in cls._dimensions)
         if not create:
             # When create is false one of the coord may be None
             if not all(key):
@@ -94,60 +93,59 @@ class Space:
 
     @classmethod
     def load(cls, points):
-        cls._db.increment(cls.convert(points))
+        return cls._db.load(cls.convert(points))
 
     @classmethod
     def convert(cls, points):
         """
-        Convert 'raw' a list of points into a list of tuple (key, values)
+        Convert a list of points into a list of tuple (key, values)
         """
         for point in points:
-            values = tuple(point[m] for m, _ in cls._measures)
-            coords = tuple(d.key(tuple(point[n])) for n, d in cls._dimensions)
+            values = tuple(point[m.name] for m in cls._measures)
+            coords = tuple(d.key(tuple(point[d.name])) for d in cls._dimensions)
             yield coords, values
 
-    @classmethod
-    def fetchmany(cls, points, skipzero=False):
-        keys = (cls.key(p, False) for p in points)
-        return cls._db.fetch(keys, skipzero=skipzero)
-
-    @classmethod
-    def fetch(cls, **point):
-        return next(cls.fetchmany((point,)))
 
     @classmethod
     def get(cls, point):
         key = cls.key(point, False)
         if key is None:
             return tuple(0 for m in cls._measures)
-        return cls._db.get(key)
+        return cls._db.dice(key)
 
     @classmethod
-    def dice(cls, point, skipzero=False):
-        points = list(cls.drill(point))
-        results = cls.fetchmany(points, skipzero=skipzero)
-        for point, res in izip(points, results):
-            point.update(res)
-            yield point
+    def dice(cls, point=[]):
+        if isinstance(point, dict):
+            point = point.items()
 
-    @classmethod
-    def drill(cls, point):
-        res = {}
-        vals = []
-        for name, dim in cls._dimensions:
-            if name not in point:
+        cube = []
+        cube_dims = []
+        cube_msrs = []
+        for name, value in point:
+            if not hasattr(cls, name):
                 continue
-            vals.append(dim.explode(point[name]))
+            dim = getattr(cls, name)
 
-        for point_tuple in product(*vals):
-            sub_point = {}
-            pos = 0
-            for name, dim in cls._dimensions:
-                if name not in point:
-                    continue
-                sub_point[name] = point_tuple[pos]
-                pos += 1
-            yield sub_point
+            if isinstance(dim, measure.Measure):
+                cube_msrs.append(dim)
+                continue
+            if not isinstance(dim, dimension.Dimension):
+                continue
+            cube_dims.append(dim)
+            coord, depth = dim.explode(value)
+            cube.append((dim, coord, depth))
+
+        cube_dims_len = len(cube_dims)
+
+        res = cls._db.dice(cube, cube_msrs)
+
+        for r in res:
+            dict_res = {}
+            for pos, d in enumerate(cube_dims):
+                dict_res[d.name] = d.get_name(r[pos])
+            for pos, m in enumerate(cube_msrs):
+                dict_res[m.name] = r[pos+cube_dims_len]
+            yield dict_res
 
 
 def build_space(data_point, name):
